@@ -30,8 +30,8 @@
 
 using namespace std;
 
-LocalHand::LocalHand(boost::shared_ptr<EngineFactory> f, GuiInterface *g, BoardInterface *b, PlayerList sl, PlayerList apl, PlayerList rpl, int id, int sP, unsigned dP, int sB,int sC)
-: myFactory(f), myGui(g),  myBoard(b), seatsList(sl), activePlayerList(apl), runningPlayerList(rpl), myBeRo(0), myID(id), startQuantityPlayers(sP), dealerPosition(dP), currentRound(0), smallBlind(sB), startCash(sC), lastPlayersTurn(-1), allInCondition(false),
+LocalHand::LocalHand(boost::shared_ptr<EngineFactory> f, GuiInterface *g, BoardInterface *b, Log *l, PlayerList sl, PlayerList apl, PlayerList rpl, int id, int sP, unsigned dP, int sB,int sC)
+: myFactory(f), myGui(g),  myBoard(b), myLog(l), seatsList(sl), activePlayerList(apl), runningPlayerList(rpl), myBeRo(0), myID(id), startQuantityPlayers(sP), dealerPosition(dP), smallBlindPosition(dP), bigBlindPosition(dP), currentRound(0), smallBlind(sB), startCash(sC), lastPlayersTurn(-1), lastActionPlayer(0), allInCondition(false),
   cardsShown(false), bettingRoundsPlayed(0)
 {
 
@@ -96,7 +96,21 @@ LocalHand::LocalHand(boost::shared_ptr<EngineFactory> f, GuiInterface *g, BoardI
 
 	// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!   DEBUGGER   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! //
 
-	if(DEBUG_MODE) {
+        if(DEBUG_MODE) {
+
+            //QSqlDatabase *mySqliteLogDb;
+
+
+
+           // myGui->getMyW()->getSession();
+
+            //QSqlDatabase * test = myGui->getMyLog()->getMySqliteLogDb();
+
+ //           if(!mySqliteLogDb->open()) {
+ //               QMessageBox::critical(0, tr("ERROR"),mySqliteLogDb->lastError().text().toUtf8().data(), QMessageBox::Cancel);
+ //           }
+
+
 
 		int temp5Array[5];
 
@@ -104,11 +118,11 @@ LocalHand::LocalHand(boost::shared_ptr<EngineFactory> f, GuiInterface *g, BoardI
 
 			case 1: {		
 
-				tempBoardArray[0] = 48;
-				tempBoardArray[1] = 10;
-				tempBoardArray[2] = 11;
-				tempBoardArray[3] = 1;
-				tempBoardArray[4] = 26;
+                                tempBoardArray[0] = 4;
+                                tempBoardArray[1] = 19;
+                                tempBoardArray[2] = 20;
+                                tempBoardArray[3] = 43;
+                                tempBoardArray[4] = 49;
 			
 				myBoard->setMyCards(tempBoardArray);
 			
@@ -121,8 +135,8 @@ LocalHand::LocalHand(boost::shared_ptr<EngineFactory> f, GuiInterface *g, BoardI
 				// player0
 				it = seatsList->begin();
 			
-				tempPlayerArray[0] = 24;
-				tempPlayerArray[1] = 45;
+                                tempPlayerArray[0] = 18;
+                                tempPlayerArray[1] = 17;
 				tempPlayerAndBoardArray[0] = tempPlayerArray[0];
 				tempPlayerAndBoardArray[1] = tempPlayerArray[1];
 			
@@ -134,8 +148,8 @@ LocalHand::LocalHand(boost::shared_ptr<EngineFactory> f, GuiInterface *g, BoardI
 				// player1
 				it++;
 		
-				tempPlayerArray[0] = 38;
-				tempPlayerArray[1] = 27;
+                                tempPlayerArray[0] = 31;
+                                tempPlayerArray[1] = 8;
 				tempPlayerAndBoardArray[0] = tempPlayerArray[0];
 				tempPlayerAndBoardArray[1] = tempPlayerArray[1];
 			
@@ -147,8 +161,8 @@ LocalHand::LocalHand(boost::shared_ptr<EngineFactory> f, GuiInterface *g, BoardI
 				// player2
 				it++;
 		
-				tempPlayerArray[0] = 49;
-				tempPlayerArray[1] = 34;
+                                tempPlayerArray[0] = 26;
+                                tempPlayerArray[1] = 27;
 				tempPlayerAndBoardArray[0] = tempPlayerArray[0];
 				tempPlayerAndBoardArray[1] = tempPlayerArray[1];
 			
@@ -160,21 +174,21 @@ LocalHand::LocalHand(boost::shared_ptr<EngineFactory> f, GuiInterface *g, BoardI
 				// player3
 				it++;
 			
-// 				tempPlayerArray[0] = 12;
-// 				tempPlayerArray[1] = 46;
-// 				tempPlayerAndBoardArray[0] = tempPlayerArray[0];
-// 				tempPlayerAndBoardArray[1] = tempPlayerArray[1];
-// 			
-// 				(*it)->setMyCards(tempPlayerArray);
-// 				(*it)->setMyCardsValueInt(myCardsValue.cardsValue(tempPlayerAndBoardArray,temp5Array));
-// 			
-// 				(*it)->setMyBestHandPosition(temp5Array);
+                                tempPlayerArray[0] = 31;
+                                tempPlayerArray[1] = 30;
+                                tempPlayerAndBoardArray[0] = tempPlayerArray[0];
+                                tempPlayerAndBoardArray[1] = tempPlayerArray[1];
+
+                                (*it)->setMyCards(tempPlayerArray);
+                                (*it)->setMyCardsValueInt(myCardsValue.cardsValue(tempPlayerAndBoardArray,temp5Array));
+
+                                (*it)->setMyBestHandPosition(temp5Array);
 
 				// player4
 				it++;
 			
-				tempPlayerArray[0] = 45;
-				tempPlayerArray[1] = 8;
+                                tempPlayerArray[0] = 50;
+                                tempPlayerArray[1] = 51;
 				tempPlayerAndBoardArray[0] = tempPlayerArray[0];
 				tempPlayerAndBoardArray[1] = tempPlayerArray[1];
 			
@@ -186,21 +200,21 @@ LocalHand::LocalHand(boost::shared_ptr<EngineFactory> f, GuiInterface *g, BoardI
 				// player5
 				it++;
 			
-// 				tempPlayerArray[0] = 43;
-// 				tempPlayerArray[1] = 30;
-// 				tempPlayerAndBoardArray[0] = tempPlayerArray[0];
-// 				tempPlayerAndBoardArray[1] = tempPlayerArray[1];
-// 			
-// 				(*it)->setMyCards(tempPlayerArray);
-// 				(*it)->setMyCardsValueInt(myCardsValue.cardsValue(tempPlayerAndBoardArray,temp5Array));
-// 			
-// 				(*it)->setMyBestHandPosition(temp5Array);
+                                tempPlayerArray[0] = 25;
+                                tempPlayerArray[1] = 24;
+                                tempPlayerAndBoardArray[0] = tempPlayerArray[0];
+                                tempPlayerAndBoardArray[1] = tempPlayerArray[1];
+
+                                (*it)->setMyCards(tempPlayerArray);
+                                (*it)->setMyCardsValueInt(myCardsValue.cardsValue(tempPlayerAndBoardArray,temp5Array));
+
+                                (*it)->setMyBestHandPosition(temp5Array);
 
 				// player6
 				it++;
 			
-				tempPlayerArray[0] = 31;
-				tempPlayerArray[1] = 19;
+                                tempPlayerArray[0] = 38;
+                                tempPlayerArray[1] = 37;
 				tempPlayerAndBoardArray[0] = tempPlayerArray[0];
 				tempPlayerAndBoardArray[1] = tempPlayerArray[1];
 			
@@ -209,6 +223,32 @@ LocalHand::LocalHand(boost::shared_ptr<EngineFactory> f, GuiInterface *g, BoardI
 			
 				(*it)->setMyBestHandPosition(temp5Array);
 
+                                // player7
+                                it++;
+
+                                tempPlayerArray[0] = 32;
+                                tempPlayerArray[1] = 33;
+                                tempPlayerAndBoardArray[0] = tempPlayerArray[0];
+                                tempPlayerAndBoardArray[1] = tempPlayerArray[1];
+
+                                (*it)->setMyCards(tempPlayerArray);
+                                (*it)->setMyCardsValueInt(myCardsValue.cardsValue(tempPlayerAndBoardArray,temp5Array));
+
+                                (*it)->setMyBestHandPosition(temp5Array);
+
+                                // player8
+                                it++;
+
+                                tempPlayerArray[0] = 19;
+                                tempPlayerArray[1] = 20;
+                                tempPlayerAndBoardArray[0] = tempPlayerArray[0];
+                                tempPlayerAndBoardArray[1] = tempPlayerArray[1];
+
+                                (*it)->setMyCards(tempPlayerArray);
+                                (*it)->setMyCardsValueInt(myCardsValue.cardsValue(tempPlayerAndBoardArray,temp5Array));
+
+                                (*it)->setMyBestHandPosition(temp5Array);
+
 
 			} break;
 			case 2: {
@@ -318,6 +358,8 @@ LocalHand::LocalHand(boost::shared_ptr<EngineFactory> f, GuiInterface *g, BoardI
 	// determine dealer, SB, BB
 	assignButtons();
 
+    myLog->logNewHandMsg(myID, dealerPosition, smallBlind, smallBlindPosition, 2*smallBlind, bigBlindPosition, seatsList);
+
 	myBeRo = myFactory->createBeRo(this, myID, dealerPosition, smallBlind);
 }
 
@@ -334,7 +376,7 @@ void LocalHand::start() {
 	it_sB = getActivePlayerIt(getCurrentBeRo()->getSmallBlindPositionId());
 	it_bB = getActivePlayerIt(getCurrentBeRo()->getBigBlindPositionId());
 	if(it_sB != getActivePlayerList()->end() && it_bB != getActivePlayerList()->end()) {
-		myGui->logNewBlindsSetsMsg((*it_sB)->getMySet(), (*it_bB)->getMySet(), (*it_sB)->getMyName().c_str(), (*it_bB)->getMyName().c_str());
+                myGui->logNewBlindsSetsMsg((*it_sB)->getMySet(), (*it_bB)->getMySet(), (*it_sB)->getMyName().c_str(), (*it_bB)->getMyName().c_str());
 	}	
 	else { LOG_ERROR(__FILE__ << " (" << __LINE__ << "): Log Error: cannot find sBID or bBID"); }
 	myGui->flushLogAtHand();	
@@ -384,14 +426,32 @@ void LocalHand::assignButtons() {
 		it = getActivePlayerIt( (*dealerPositionIt)->getMyUniqueID() );
 		if(it != activePlayerList->end() ) {
 			nextActivePlayerFound = true;
-			if(activePlayerList->size() > 2) (*it)->setMyButton(2); //small blind normal
-			else (*it)->setMyButton(3); //big blind in heads up
+                        if(activePlayerList->size() > 2) {
+                            //small blind normal
+                            (*it)->setMyButton(2);
+                            smallBlindPosition = (*it)->getMyUniqueID();
+                        } else {
+                            //big blind in heads up
+                            (*it)->setMyButton(3);
+                            bigBlindPosition = (*it)->getMyUniqueID();
+                            // lastPlayerAction for showing cards
+                        }
+
+                        // first player after dealer have to show his cards first (in showdown)
+                        lastActionPlayer = (*it)->getMyUniqueID();
 
 			it++;
 			if(it == activePlayerList->end()) it = activePlayerList->begin();
 
-			if(activePlayerList->size() > 2) (*it)->setMyButton(3); //big blind normal
-			else (*it)->setMyButton(2); //small blind in heads up
+                        if(activePlayerList->size() > 2) {
+                             //big blind normal
+                            (*it)->setMyButton(3);
+                            bigBlindPosition = (*it)->getMyUniqueID();
+                        } else {
+                            //small blind in heads up
+                            (*it)->setMyButton(2);
+                            smallBlindPosition = (*it)->getMyUniqueID();
+                        }
 
 			break;
 		}
@@ -401,6 +461,8 @@ void LocalHand::assignButtons() {
 		throw LocalException(__FILE__, __LINE__, ERR_NEXT_ACTIVE_PLAYER_NOT_FOUND);
 	}
 
+//        cout << "lAP-Button: " << lastActionPlayer << endl;
+
 	//do sets --> TODO switch?
 	for (it_c=runningPlayerList->begin(); it_c!=runningPlayerList->end(); it_c++) { 
 
