@@ -20,7 +20,11 @@
 #ifndef NEWGAMEDIALOGIMPL_H
 #define NEWGAMEDIALOGIMPL_H
 
-#include "ui_newgamedialog.h"
+//--- GUI_800x480 ---
+#include "ui_newgamedialog_800x480.h"
+//---
+// #include "ui_newgamedialog.h"
+//---
 
 #include <QtGui>
 #include <QtCore>
