@@ -20,7 +20,11 @@
 #ifndef STARTNETWORKGAMEDIALOGIMPL_H
 #define STARTNETWORKGAMEDIALOGIMPL_H
 
-#include "ui_startnetworkgamedialog.h"
+//--- GUI_800x480
+#include "ui_startnetworkgamedialog_800x480.h"
+//---
+// #include "ui_startnetworkgamedialog.h"
+//---
 
 #include <QtGui>
 #include <QtCore>
@@ -42,15 +46,15 @@ public:
 public slots:
 
 	void setMyW ( gameTableImpl* theValue ) { myW = theValue; }
-	
+	ChatTools *getMyChat() { return myChat; }	
 
 	void startGame();
 	void cancel();
 	void refresh(int actionID);
 	void accept();
 	void reject();
-	void joinedNetworkGame(unsigned playerId, QString playerName, int rights);
-	void addConnectedPlayer(unsigned playerId, QString playerName, int rights);
+        void joinedNetworkGame(unsigned playerId, QString playerName, bool admin);
+        void addConnectedPlayer(unsigned playerId, QString playerName, bool admin);
 	void updatePlayer(unsigned playerId, QString newPlayerName);
 	void removePlayer(unsigned playerId, QString playerName);
 	void newGameAdmin(unsigned playerId, QString playerName);
@@ -62,12 +66,9 @@ public slots:
 	void checkPlayerQuantity();
 	void clearDialog();
 
-	void receiveChatMsg(QString playerName, QString message);
-
 	void keyPressEvent ( QKeyEvent*);
 	bool eventFilter(QObject *obj, QEvent *event);
 
-
 	void setMaxPlayerNumber ( int theValue ) { maxPlayerNumber = theValue; label_maxPlayerNumber->setText(QString::number(theValue,10)); }
 	int getMaxPlayerNumber() const { return maxPlayerNumber; }
 
@@ -80,7 +81,7 @@ private:
 	int maxPlayerNumber;
 	int keyUpDownChatCounter;
 	unsigned myPlayerId;
-	bool isAdmin;
+        bool isAdmin;
 	ConfigFile *myConfig;
 	boost::shared_ptr<Session> mySession;
 	ChatTools *myChat;
