@@ -24,7 +24,6 @@
 #include <boost/shared_ptr.hpp>
 #include <string>
 
-#include <playerdata.h>
 #include <game_defs.h>
 #include <gamedata.h>
 
@@ -51,20 +50,32 @@ public:
 	virtual void SignalNetClientGameListPlayerLeft(unsigned gameId, unsigned playerId) = 0;
 
 	virtual void SignalNetClientGameStart(boost::shared_ptr<Game> game) = 0;
-	virtual void SignalNetClientSelfJoined(unsigned playerId, const std::string &playerName, PlayerRights rights) = 0;
-	virtual void SignalNetClientPlayerJoined(unsigned playerId, const std::string &playerName, PlayerRights rights) = 0;
+	virtual void SignalNetClientSelfJoined(unsigned playerId, const std::string &playerName, bool isGameAdmin) = 0;
+	virtual void SignalNetClientPlayerJoined(unsigned playerId, const std::string &playerName, bool isGameAdmin) = 0;
 	virtual void SignalNetClientPlayerChanged(unsigned playerId, const std::string &newPlayerName) = 0;
 	virtual void SignalNetClientPlayerLeft(unsigned playerId, const std::string &playerName, int removeReason) = 0;
 	virtual void SignalNetClientNewGameAdmin(unsigned playerId, const std::string &playerName) = 0;
 
-	virtual void SignalNetClientChatMsg(const std::string &playerName, const std::string &msg) = 0;
+	virtual void SignalNetClientGameChatMsg(const std::string &playerName, const std::string &msg) = 0;
+	virtual void SignalNetClientLobbyChatMsg(const std::string &playerName, const std::string &msg) = 0;
 	virtual void SignalNetClientMsgBox(const std::string &msg) = 0;
 	virtual void SignalNetClientWaitDialog() = 0;
-	
-	virtual void SignalNetClientServerListAdd(unsigned serverId) =0;
-	virtual void SignalNetClientServerListClear() =0;
-	virtual void SignalNetClientServerListShow() =0;
+        virtual void SignalNetClientWarningAutoFoldInRankingGame(unsigned remainingAutoFolds) = 0;
 
+	virtual void SignalNetClientServerListAdd(unsigned serverId) = 0;
+	virtual void SignalNetClientServerListClear() = 0;
+	virtual void SignalNetClientServerListShow() = 0;
+
+	virtual void SignalNetClientLoginShow() = 0;
+	virtual void SignalNetClientPostRiverShowCards(unsigned playerId) = 0;
+
+	virtual void SignalLobbyPlayerJoined(unsigned playerId, const std::string &nickName) = 0;
+	virtual void SignalLobbyPlayerKicked(const std::string &nickName, const std::string &byWhom, const std::string &reason) = 0;
+	virtual void SignalLobbyPlayerLeft(unsigned playerId) = 0;
+
+	virtual void SignalSelfGameInvitation(unsigned gameId, unsigned playerIdFrom) = 0;
+	virtual void SignalPlayerGameInvitation(unsigned gameId, unsigned playerIdWho, unsigned playerIdFrom) = 0;
+	virtual void SignalRejectedGameInvitation(unsigned gameId, unsigned playerIdWho, DenyGameInvitationReason reason) = 0;
 };
 
 #endif
