@@ -27,16 +27,19 @@
 #include <net/downloaderthread.h>
 #include <net/clientexception.h>
 #include <net/socket_msg.h>
+#include <net/net_helper.h>
 #include <core/avatarmanager.h>
 #include <core/loghelper.h>
 #include <clientenginefactory.h>
 #include <game.h>
+#include <log.h>
 #include <qttoolsinterface.h>
 
 #include <boost/lambda/lambda.hpp>
 #include <sstream>
 #include <memory>
 #include <cassert>
+#include <gsasl.h>
 
 #define TEMP_AVATAR_FILENAME "avatar.tmp"
 #define CLIENT_AVATAR_LOOP_MSEC	100
@@ -69,6 +72,7 @@ ClientThread::ClientThread(GuiInterface &gui, AvatarManager &avatarManager)
   m_curGameId(0), m_curGameNum(1), m_guiPlayerId(0), m_sessionEstablished(false),
   m_stateTimer(*m_ioService), m_avatarTimer(*m_ioService)
 {
+	m_clientLog.reset(new Log("", 0));
 	m_context.reset(new ClientContext);
 	m_receiver.reset(new ReceiverHelper);
 	myQtToolsInterface.reset(CreateQtToolsWrapper());
@@ -84,9 +88,8 @@ void
 ClientThread::Init(
 	const string &serverAddress, const string &serverListUrl,
 	bool useServerList, unsigned serverPort, bool ipv6, bool sctp,
-	const string &avatarServerAddress, const string &pwd,
-	const string &playerName, const string &avatarFile,
-	const string &cacheDir)
+	const string &avatarServerAddress, const string &playerName,
+	const string &avatarFile, const string &cacheDir)
 {
 	if (IsRunning())
 	{
@@ -103,7 +106,6 @@ ClientThread::Init(
 	context.SetUseServerList(useServerList);
 	context.SetServerPort(serverPort);
 	context.SetAvatarServerAddr(avatarServerAddress);
-	context.SetPassword(pwd);
 	context.SetPlayerName(playerName);
 	context.SetAvatarFile(avatarFile);
 	context.SetCacheDir(cacheDir);
@@ -119,18 +121,22 @@ ClientThread::SignalTermination()
 void
 ClientThread::SendKickPlayer(unsigned playerId)
 {
-	boost::shared_ptr<NetPacket> request(new NetPacketKickPlayer);
-	NetPacketKickPlayer::Data requestData;
-	requestData.playerId = playerId;
-	static_cast<NetPacketKickPlayer *>(request.get())->SetData(requestData);
-	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), request));
+	boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+	packet->GetMsg()->present = PokerTHMessage_PR_kickPlayerRequestMessage;
+	KickPlayerRequestMessage_t *netKick = &packet->GetMsg()->choice.kickPlayerRequestMessage;
+	netKick->gameId = GetGameId();
+	netKick->playerId = playerId;
+	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), packet));
 }
 
 void
 ClientThread::SendLeaveCurrentGame()
 {
-	boost::shared_ptr<NetPacket> request(new NetPacketLeaveCurrentGame);
-	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), request));
+	boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+	packet->GetMsg()->present = PokerTHMessage_PR_leaveGameRequestMessage;
+	LeaveGameRequestMessage_t *netLeave = &packet->GetMsg()->choice.leaveGameRequestMessage;
+	netLeave->gameId = GetGameId();
+	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), packet));
 }
 
 void
@@ -138,17 +144,12 @@ ClientThread::SendStartEvent(bool fillUpWithCpuPlayers)
 {
 	// Warning: This function is called in the context of the GUI thread.
 	// Create a network packet for the server start event.
-	boost::shared_ptr<NetPacket> start(new NetPacketStartEvent);
-	NetPacketStartEvent::Data startData;
-	startData.fillUpWithCpuPlayers = fillUpWithCpuPlayers;
-	try
-	{
-		static_cast<NetPacketStartEvent *>(start.get())->SetData(startData);
-		m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), start));
-	} catch (const NetException &e)
-	{
-		LOG_ERROR("ClientThread::SendStartEvent: " << e.what());
-	}
+	boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+	packet->GetMsg()->present = PokerTHMessage_PR_startEventMessage;
+	StartEventMessage_t *netStartEvent = &packet->GetMsg()->choice.startEventMessage;
+	netStartEvent->gameId = GetGameId();
+	netStartEvent->fillWithComputerPlayers = fillUpWithCpuPlayers;
+	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), packet));
 }
 
 void
@@ -156,44 +157,77 @@ ClientThread::SendPlayerAction()
 {
 	// Warning: This function is called in the context of the GUI thread.
 	// Create a network packet containing the current player action.
-	boost::shared_ptr<NetPacket> action(new NetPacketPlayersAction);
-	NetPacketPlayersAction::Data actionData;
+	boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+	packet->GetMsg()->present = PokerTHMessage_PR_myActionRequestMessage;
+	MyActionRequestMessage_t *netMyAction = &packet->GetMsg()->choice.myActionRequestMessage;
+	netMyAction->gameId = GetGameId();
 	boost::shared_ptr<PlayerInterface> myPlayer = GetGame()->getSeatsList()->front();
-	actionData.gameState = static_cast<GameState>(GetGame()->getCurrentHand()->getCurrentRound());
-	actionData.playerAction = static_cast<PlayerAction>(myPlayer->getMyAction());
+	netMyAction->handNum = GetGame()->getCurrentHandID();
+	netMyAction->gameState = GetGame()->getCurrentHand()->getCurrentRound();
+	netMyAction->myAction = myPlayer->getMyAction();
 	// Only send last bet if not fold/checked.
-	if (actionData.playerAction != PLAYER_ACTION_FOLD && actionData.playerAction != PLAYER_ACTION_CHECK)
-		actionData.playerBet = myPlayer->getMyLastRelativeSet();
+	if (myPlayer->getMyAction() != PLAYER_ACTION_FOLD && myPlayer->getMyAction() != PLAYER_ACTION_CHECK)
+		netMyAction->myRelativeBet = myPlayer->getMyLastRelativeSet();
 	else
-		actionData.playerBet = 0;
-	try
-	{
-		static_cast<NetPacketPlayersAction *>(action.get())->SetData(actionData);
-		// Just dump the packet.
-		m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), action));
-	} catch (const NetException &e)
-	{
-		LOG_ERROR("ClientThread::SendPlayerAction: " << e.what());
-	}
+		netMyAction->myRelativeBet = 0;
+	// Just dump the packet.
+	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), packet));
 }
 
 void
-ClientThread::SendChatMessage(const std::string &msg)
+ClientThread::SendGameChatMessage(const std::string &msg)
 {
 	// Warning: This function is called in the context of the GUI thread.
 	// Create a network packet containing the chat message.
-	boost::shared_ptr<NetPacket> chat(new NetPacketSendChatText);
-	NetPacketSendChatText::Data chatData;
-	chatData.text = msg;
-	try
-	{
-		static_cast<NetPacketSendChatText *>(chat.get())->SetData(chatData);
-		// Just dump the packet.
-		m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), chat));
-	} catch (const NetException &e)
-	{
-		LOG_ERROR("ClientThread::SendChatMessage: " << e.what());
-	}
+	boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+	packet->GetMsg()->present = PokerTHMessage_PR_chatRequestMessage;
+	ChatRequestMessage_t *netChat = &packet->GetMsg()->choice.chatRequestMessage;
+	OCTET_STRING_fromBuf(&netChat->chatText,
+						 msg.c_str(),
+						 msg.length());
+
+	netChat->chatRequestType.present = chatRequestType_PR_chatRequestTypeGame;
+	netChat->chatRequestType.choice.chatRequestTypeGame.gameId = GetGameId();
+
+	// Just dump the packet.
+	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), packet));
+}
+
+void
+ClientThread::SendLobbyChatMessage(const std::string &msg)
+{
+	// Warning: This function is called in the context of the GUI thread.
+	// Create a network packet containing the chat message.
+	boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+	packet->GetMsg()->present = PokerTHMessage_PR_chatRequestMessage;
+	ChatRequestMessage_t *netChat = &packet->GetMsg()->choice.chatRequestMessage;
+	OCTET_STRING_fromBuf(&netChat->chatText,
+						 msg.c_str(),
+						 msg.length());
+
+	netChat->chatRequestType.present = chatRequestType_PR_chatRequestTypeLobby;
+
+	// Just dump the packet.
+	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), packet));
+}
+
+void
+ClientThread::SendPrivateChatMessage(unsigned targetPlayerId, const std::string &msg)
+{
+	// Warning: This function is called in the context of the GUI thread.
+	// Create a network packet containing the chat message.
+	boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+	packet->GetMsg()->present = PokerTHMessage_PR_chatRequestMessage;
+	ChatRequestMessage_t *netChat = &packet->GetMsg()->choice.chatRequestMessage;
+	OCTET_STRING_fromBuf(&netChat->chatText,
+						 msg.c_str(),
+						 msg.length());
+
+	netChat->chatRequestType.present = chatRequestType_PR_chatRequestTypePrivate;
+	netChat->chatRequestType.choice.chatRequestTypePrivate.targetPlayerId = targetPlayerId;
+
+	// Just dump the packet.
+	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), packet));
 }
 
 void
@@ -201,18 +235,21 @@ ClientThread::SendJoinFirstGame(const std::string &password)
 {
 	// Warning: This function is called in the context of the GUI thread.
 	// Create a network packet to request joining a game.
-	boost::shared_ptr<NetPacket> join(new NetPacketJoinGame);
-	NetPacketJoinGame::Data joinData;
-	joinData.gameId = 1;
-	joinData.password = password;
-	try
+	boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+	packet->GetMsg()->present = PokerTHMessage_PR_joinGameRequestMessage;
+	JoinGameRequestMessage_t *netJoinGame = &packet->GetMsg()->choice.joinGameRequestMessage;
+	if (!password.empty())
 	{
-		static_cast<NetPacketJoinGame *>(join.get())->SetData(joinData);
-		m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), join));
-	} catch (const NetException &e)
-	{
-		LOG_ERROR("ClientThread::SendJoinFirstGame: " << e.what());
+		netJoinGame->password = OCTET_STRING_new_fromBuf(
+			&asn_DEF_UTF8String,
+			password.c_str(),
+			password.length());
 	}
+	netJoinGame->joinGameAction.present = joinGameAction_PR_joinExistingGame;
+
+	JoinExistingGame_t *joinExisting = &netJoinGame->joinGameAction.choice.joinExistingGame;
+	joinExisting->gameId = 1;
+	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), packet));
 }
 
 void
@@ -220,18 +257,21 @@ ClientThread::SendJoinGame(unsigned gameId, const std::string &password)
 {
 	// Warning: This function is called in the context of the GUI thread.
 	// Create a network packet to request joining a game.
-	boost::shared_ptr<NetPacket> join(new NetPacketJoinGame);
-	NetPacketJoinGame::Data joinData;
-	joinData.password = password;
-	joinData.gameId = gameId;
-	try
+	boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+	packet->GetMsg()->present = PokerTHMessage_PR_joinGameRequestMessage;
+	JoinGameRequestMessage_t *netJoinGame = &packet->GetMsg()->choice.joinGameRequestMessage;
+	if (!password.empty())
 	{
-		static_cast<NetPacketJoinGame *>(join.get())->SetData(joinData);
-		m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), join));
-	} catch (const NetException &e)
-	{
-		LOG_ERROR("ClientThread::SendJoinGame: " << e.what());
+		netJoinGame->password = OCTET_STRING_new_fromBuf(
+			&asn_DEF_UTF8String,
+			password.c_str(),
+			password.length());
 	}
+	netJoinGame->joinGameAction.present = joinGameAction_PR_joinExistingGame;
+
+	JoinExistingGame_t *joinExisting = &netJoinGame->joinGameAction.choice.joinExistingGame;
+	joinExisting->gameId = gameId;
+	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), packet));
 }
 
 void
@@ -239,50 +279,107 @@ ClientThread::SendCreateGame(const GameData &gameData, const std::string &name,
 {
 	// Warning: This function is called in the context of the GUI thread.
 	// Create a network packet to request creating a new game.
-	boost::shared_ptr<NetPacket> create(new NetPacketCreateGame);
-	NetPacketCreateGame::Data createData;
-	createData.gameData = gameData;
-	createData.gameName = name;
-	createData.password = password;
-	try
+	boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+	packet->GetMsg()->present = PokerTHMessage_PR_joinGameRequestMessage;
+	JoinGameRequestMessage_t *netJoinGame = &packet->GetMsg()->choice.joinGameRequestMessage;
+	if (!password.empty())
 	{
-		static_cast<NetPacketCreateGame *>(create.get())->SetData(createData);
-		m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), create));
-	} catch (const NetException &e)
-	{
-		LOG_ERROR("ClientThread::SendCreateGame: " << e.what());
+		netJoinGame->password = OCTET_STRING_new_fromBuf(
+			&asn_DEF_UTF8String,
+			password.c_str(),
+			password.length());
 	}
+	netJoinGame->joinGameAction.present = joinGameAction_PR_joinNewGame;
+
+	JoinNewGame_t *joinNew = &netJoinGame->joinGameAction.choice.joinNewGame;
+	NetPacket::SetGameData(gameData, &joinNew->gameInfo);
+	OCTET_STRING_fromBuf(&joinNew->gameInfo.gameName,
+						 name.c_str(),
+						 name.length());
+	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), packet));
 }
 
 void
 ClientThread::SendResetTimeout()
 {
-	boost::shared_ptr<NetPacket> reset(new NetPacketResetTimeout);
-	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), reset));
+	boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+	packet->GetMsg()->present = PokerTHMessage_PR_resetTimeoutMessage;
+	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), packet));
 }
 
 void
 ClientThread::SendAskKickPlayer(unsigned playerId)
 {
-	boost::shared_ptr<NetPacket> ask(new NetPacketAskKickPlayer);
-	NetPacketAskKickPlayer::Data askData;
-	askData.playerId = playerId;
-	static_cast<NetPacketAskKickPlayer *>(ask.get())->SetData(askData);
-	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), ask));
+	boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+	packet->GetMsg()->present = PokerTHMessage_PR_askKickPlayerMessage;
+	AskKickPlayerMessage_t *netAsk = &packet->GetMsg()->choice.askKickPlayerMessage;
+	netAsk->gameId = GetGameId();
+	netAsk->playerId = playerId;
+	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), packet));
 }
 
 void
 ClientThread::SendVoteKick(bool doKick)
 {
-	boost::shared_ptr<NetPacket> vote(new NetPacketVoteKickPlayer);
-	NetPacketVoteKickPlayer::Data voteData;
+	boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+	packet->GetMsg()->present = PokerTHMessage_PR_voteKickRequestMessage;
+	VoteKickRequestMessage_t *netVote = &packet->GetMsg()->choice.voteKickRequestMessage;
+	netVote->gameId = GetGameId();
 	{
 		boost::mutex::scoped_lock lock(m_curPetitionIdMutex);
-		voteData.petitionId = m_curPetitionId;
+		netVote->petitionId = m_curPetitionId;
+	}
+	netVote->voteKick = doKick;
+	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), packet));
+}
+
+void
+ClientThread::SendShowMyCards()
+{
+	boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+	packet->GetMsg()->present = PokerTHMessage_PR_showMyCardsRequestMessage;
+	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), packet));
+}
+
+void
+ClientThread::SendInvitePlayerToCurrentGame(unsigned playerId)
+{
+	boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+	packet->GetMsg()->present = PokerTHMessage_PR_invitePlayerToGameMessage;
+	InvitePlayerToGameMessage_t *netInvite = &packet->GetMsg()->choice.invitePlayerToGameMessage;
+	netInvite->gameId = GetGameId();
+	netInvite->playerId = playerId;
+	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), packet));
+}
+
+void
+ClientThread::SendRejectGameInvitation(unsigned gameId, DenyGameInvitationReason reason)
+{
+	boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+	packet->GetMsg()->present = PokerTHMessage_PR_rejectGameInvitationMessage;
+	RejectGameInvitationMessage_t *netReject = &packet->GetMsg()->choice.rejectGameInvitationMessage;
+	netReject->gameId = gameId;
+	netReject->myRejectReason = static_cast<RejectGameInvReason_t>(reason);
+	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), packet));
+}
+
+void
+ClientThread::SendReportAvatar(unsigned reportedPlayerId, const std::string &avatarHash)
+{
+	boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+	packet->GetMsg()->present = PokerTHMessage_PR_reportAvatarMessage;
+	ReportAvatarMessage_t *netReport = &packet->GetMsg()->choice.reportAvatarMessage;
+	netReport->reportedPlayerId = reportedPlayerId;
+	MD5Buf tmpMD5;
+	if (tmpMD5.FromString(avatarHash) && !tmpMD5.IsZero())
+	{
+		OCTET_STRING_fromBuf(
+			&netReport->reportedAvatar,
+			(const char *)tmpMD5.GetData(),
+			MD5_DATA_SIZE);
+
+		m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), packet));
 	}
-	voteData.vote = doKick ? KICK_VOTE_IN_FAVOUR : KICK_VOTE_AGAINST;
-	static_cast<NetPacketVoteKickPlayer *>(vote.get())->SetData(voteData);
-	m_ioService->post(boost::bind(&ClientThread::SendSessionPacket, shared_from_this(), vote));
 }
 
 void
@@ -331,6 +428,15 @@ ClientThread::SelectServer(unsigned serverId)
 	m_selectedServerId = serverId;
 }
 
+void
+ClientThread::SetLogin(const std::string &userName, const std::string &password, bool isGuest)
+{
+	boost::mutex::scoped_lock lock(m_loginDataMutex);
+	m_loginData.userName = userName;
+	m_loginData.password = password;
+	m_loginData.isGuest = isGuest;
+}
+
 ServerInfo
 ClientThread::GetServerInfo(unsigned serverId) const
 {
@@ -393,6 +499,35 @@ ClientThread::GetPlayerIdFromName(const string &playerName, unsigned &playerId)
 	return retVal;
 }
 
+unsigned
+ClientThread::GetGameIdOfPlayer(unsigned playerId) const
+{
+	unsigned gameId = 0; // Default: no game (invalid id).
+
+	// Iterate through all games to find the player.
+	boost::mutex::scoped_lock lock(m_gameInfoMapMutex);
+	GameInfoMap::const_iterator i = m_gameInfoMap.begin();
+	GameInfoMap::const_iterator i_end = m_gameInfoMap.end();
+	while (i != i_end)
+	{
+		PlayerIdList::const_iterator j = (*i).second.players.begin();
+		PlayerIdList::const_iterator j_end = (*i).second.players.end();
+		while (j != j_end)
+		{
+			if (playerId == *j)
+			{
+				gameId = (*i).first;
+				break;
+			}
+			++j;
+		}
+		if (gameId)
+			break;
+		++i;
+	}
+	return gameId;
+}
+
 ClientCallback &
 ClientThread::GetCallback()
 {
@@ -414,16 +549,17 @@ ClientThread::GetAvatarManager()
 void
 ClientThread::Main()
 {
-	// Start sub-threads.
-	m_avatarDownloader.reset(new DownloaderThread);
-	m_avatarDownloader->Run();
-	SetState(CLIENT_INITIAL_STATE::Instance());
-	RegisterTimers();
-
 	// Main loop.
 	boost::asio::io_service::work ioWork(*m_ioService);
 	try
 	{
+		InitAuthContext();
+		// Start sub-threads.
+		m_avatarDownloader.reset(new DownloaderThread);
+		m_avatarDownloader->Run();
+		SetState(CLIENT_INITIAL_STATE::Instance());
+		RegisterTimers();
+
 		boost::asio::io_service::work ioWork(*m_ioService);
 		m_ioService->run(); // Will only be aborted asynchronously.
 
@@ -433,7 +569,7 @@ ClientThread::Main()
 	}
 	// Close the socket.
 	boost::system::error_code ec;
-	GetContext().GetSessionData()->GetAsioSocket()->close();
+	GetContext().GetSessionData()->GetAsioSocket()->close(ec);
 	// Set a state which does not do anything.
 	SetState(CLIENT_FINAL_STATE::Instance());
 	// Cancel timers.
@@ -442,6 +578,8 @@ ClientThread::Main()
 	// Terminate sub-threads.
 	m_avatarDownloader->SignalTermination();
 	m_avatarDownloader->Join(DOWNLOADER_THREAD_TERMINATE_TIMEOUT);
+
+	ClearAuthContext();
 }
 
 void
@@ -460,6 +598,27 @@ ClientThread::CancelTimers()
 	m_avatarTimer.cancel();
 }
 
+void
+ClientThread::InitAuthContext()
+{
+	int res = gsasl_init(&m_authContext);
+	if (res != GSASL_OK)
+		throw ClientException(__FILE__, __LINE__, ERR_NET_GSASL_INIT_FAILED, 0);
+
+	if (!gsasl_server_support_p(m_authContext, "SCRAM-SHA-1"))
+	{
+		gsasl_done(m_authContext);
+		throw ClientException(__FILE__, __LINE__, ERR_NET_GSASL_NO_SCRAM, 0);
+	}
+}
+
+void
+ClientThread::ClearAuthContext()
+{
+	gsasl_done(m_authContext);
+	m_authContext = NULL;
+}
+
 void
 ClientThread::InitGame()
 {
@@ -469,9 +628,13 @@ ClientThread::InitGame()
 	MapPlayerDataList();
 	if (GetPlayerDataList().size() != (unsigned)GetStartData().numberOfPlayers)
 		throw ClientException(__FILE__, __LINE__, ERR_NET_INVALID_PLAYER_COUNT, 0);
-	m_game.reset(new Game(&m_gui, factory, GetPlayerDataList(), GetGameData(), GetStartData(), m_curGameNum++));
-	// Initialize GUI speed.
-	GetGui().initGui(GetGameData().guiSpeed);
+	m_game.reset(new Game(&m_gui, factory, GetPlayerDataList(), GetGameData(), GetStartData(), m_curGameNum++, m_clientLog.get()));
+	// Initialize Minimum GUI speed.
+	int minimumGuiSpeed = 1;
+	if(GetGameData().delayBetweenHandsSec < 11) {
+		minimumGuiSpeed = 12-GetGameData().delayBetweenHandsSec;
+	}
+	GetGui().initGui(minimumGuiSpeed);
 	// Signal start of game to GUI.
 	GetCallback().SignalNetClientGameStart(m_game);
 }
@@ -523,11 +686,11 @@ ClientThread::RequestPlayerInfo(unsigned id, bool requestAvatar)
 {
 	if (find(m_playerInfoRequestList.begin(), m_playerInfoRequestList.end(), id) == m_playerInfoRequestList.end())
 	{
-		boost::shared_ptr<NetPacket> req(new NetPacketRetrievePlayerInfo);
-		NetPacketRetrievePlayerInfo::Data reqData;
-		reqData.playerId = id;
-		static_cast<NetPacketRetrievePlayerInfo *>(req.get())->SetData(reqData);
-		GetSender().Send(GetContext().GetSessionData(), req);
+		boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+		packet->GetMsg()->present = PokerTHMessage_PR_playerInfoRequestMessage;
+		PlayerInfoRequestMessage_t *netPlayerInfo = &packet->GetMsg()->choice.playerInfoRequestMessage;
+		netPlayerInfo->playerId = id;
+		GetSender().Send(GetContext().GetSessionData(), packet);
 
 		m_playerInfoRequestList.push_back(id);
 
@@ -613,7 +776,7 @@ ClientThread::SetNewGameAdmin(unsigned id)
 	boost::shared_ptr<PlayerData> playerData = GetPlayerDataByUniqueId(id);
 	if (playerData.get())
 	{
-		playerData->SetRights(PLAYER_RIGHTS_ADMIN);
+		playerData->SetGameAdmin(true);
 		GetCallback().SignalNetClientNewGameAdmin(id, playerData->GetName());
 	}
 }
@@ -637,21 +800,23 @@ ClientThread::RetrieveAvatarIfNeeded(unsigned id, const PlayerInfo &info)
 			}
 			else
 			{
-				boost::shared_ptr<NetPacket> retrieveAvatar(new NetPacketRetrieveAvatar);
-				NetPacketRetrieveAvatar::Data retrieveAvatarData;
-				retrieveAvatarData.requestId = id;
-				retrieveAvatarData.avatar = info.avatar;
-				static_cast<NetPacketRetrieveAvatar *>(retrieveAvatar.get())->SetData(retrieveAvatarData);
-				GetSender().Send(GetContext().GetSessionData(), retrieveAvatar);
+				boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+				packet->GetMsg()->present = PokerTHMessage_PR_avatarRequestMessage;
+				AvatarRequestMessage_t *netAvatar = &packet->GetMsg()->choice.avatarRequestMessage;
+				netAvatar->requestId = id;
+				OCTET_STRING_fromBuf(&netAvatar->avatar,
+									 (const char *)info.avatar.GetData(),
+									 MD5_DATA_SIZE);
+				GetSender().Send(GetContext().GetSessionData(), packet);
 			}
 		}
 	}
 }
 
 void
-ClientThread::AddTempAvatarData(unsigned playerId, unsigned avatarSize, AvatarFileType type)
+ClientThread::AddTempAvatarFile(unsigned playerId, unsigned avatarSize, AvatarFileType type)
 {
-	boost::shared_ptr<AvatarData> tmpAvatar(new AvatarData);
+	boost::shared_ptr<AvatarFile> tmpAvatar(new AvatarFile);
 	tmpAvatar->fileData.reserve(avatarSize);
 	tmpAvatar->fileType = type;
 	tmpAvatar->reportedSize = avatarSize;
@@ -660,9 +825,9 @@ ClientThread::AddTempAvatarData(unsigned playerId, unsigned avatarSize, AvatarFi
 }
 
 void
-ClientThread::StoreInTempAvatarData(unsigned playerId, const vector<unsigned char> &data)
+ClientThread::StoreInTempAvatarFile(unsigned playerId, const vector<unsigned char> &data)
 {
-	AvatarDataMap::iterator pos = m_tempAvatarMap.find(playerId);
+	AvatarFileMap::iterator pos = m_tempAvatarMap.find(playerId);
 	if (pos == m_tempAvatarMap.end())
 		throw ClientException(__FILE__, __LINE__, ERR_NET_INVALID_REQUEST_ID, 0);
 	// We trust the server (concerning size of the data).
@@ -670,33 +835,33 @@ ClientThread::StoreInTempAvatarData(unsigned playerId, const vector<unsigned cha
 }
 
 void
-ClientThread::CompleteTempAvatarData(unsigned playerId)
+ClientThread::CompleteTempAvatarFile(unsigned playerId)
 {
-	AvatarDataMap::iterator pos = m_tempAvatarMap.find(playerId);
+	AvatarFileMap::iterator pos = m_tempAvatarMap.find(playerId);
 	if (pos == m_tempAvatarMap.end())
 		throw ClientException(__FILE__, __LINE__, ERR_NET_INVALID_REQUEST_ID, 0);
-	boost::shared_ptr<AvatarData> tmpAvatar = pos->second;
+	boost::shared_ptr<AvatarFile> tmpAvatar = pos->second;
 	unsigned avatarSize = (unsigned)tmpAvatar->fileData.size();
 	if (avatarSize != tmpAvatar->reportedSize)
 		LOG_ERROR("Client received invalid avatar file size!");
 	else
-		PassAvatarDataToManager(playerId, tmpAvatar);
+		PassAvatarFileToManager(playerId, tmpAvatar);
 
 	// Free memory.
 	m_tempAvatarMap.erase(pos);
 }
 
 void
-ClientThread::PassAvatarDataToManager(unsigned playerId, boost::shared_ptr<AvatarData> avatarData)
+ClientThread::PassAvatarFileToManager(unsigned playerId, boost::shared_ptr<AvatarFile> AvatarFile)
 {
 	PlayerInfo tmpPlayerInfo;
 	if (!GetCachedPlayerInfo(playerId, tmpPlayerInfo))
 		LOG_ERROR("Client received invalid player id!");
 	else
 	{
-		if (avatarData->fileType == AVATAR_FILE_TYPE_UNKNOWN)
-			avatarData->fileType = tmpPlayerInfo.avatarType;
-		if (!GetAvatarManager().StoreAvatarInCache(tmpPlayerInfo.avatar, avatarData->fileType, &avatarData->fileData[0], avatarData->reportedSize, false))
+		if (AvatarFile->fileType == AVATAR_FILE_TYPE_UNKNOWN)
+			AvatarFile->fileType = tmpPlayerInfo.avatarType;
+		if (!GetAvatarManager().StoreAvatarInCache(tmpPlayerInfo.avatar, AvatarFile->fileType, &AvatarFile->fileData[0], AvatarFile->reportedSize, false))
 			LOG_ERROR("Failed to store avatar in cache directory.");
 
 		// Update player info, but never re-request avatar.
@@ -726,10 +891,10 @@ ClientThread::TimerCheckAvatarDownloads(const boost::system::error_code& ec)
 		if (m_avatarDownloader && m_avatarDownloader->HasDownloadResult())
 		{
 			unsigned playerId;
-			boost::shared_ptr<AvatarData> tmpAvatar(new AvatarData);
+			boost::shared_ptr<AvatarFile> tmpAvatar(new AvatarFile);
 			m_avatarDownloader->GetDownloadResult(playerId, tmpAvatar->fileData);
 			tmpAvatar->reportedSize = tmpAvatar->fileData.size();
-			PassAvatarDataToManager(playerId, tmpAvatar);
+			PassAvatarFileToManager(playerId, tmpAvatar);
 		}
 		m_avatarTimer.expires_from_now(
 			boost::posix_time::milliseconds(CLIENT_AVATAR_LOOP_MSEC));
@@ -745,8 +910,11 @@ ClientThread::UnsubscribeLobbyMsg()
 	if (GetContext().GetSubscribeLobbyMsg())
 	{
 		// Send unsubscribe request.
-		boost::shared_ptr<NetPacket> unsubscr(new NetPacketUnsubscribeGameList);
-		GetSender().Send(GetContext().GetSessionData(), unsubscr);
+		boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+		packet->GetMsg()->present = PokerTHMessage_PR_subscriptionRequestMessage;
+		SubscriptionRequestMessage_t *netRequest = &packet->GetMsg()->choice.subscriptionRequestMessage;
+		netRequest->subscriptionAction = subscriptionAction_unsubscribeGameList;
+		GetSender().Send(GetContext().GetSessionData(), packet);
 		GetContext().SetSubscribeLobbyMsg(false);
 	}
 }
@@ -759,8 +927,11 @@ ClientThread::ResubscribeLobbyMsg()
 		// Clear game info map as it is outdated.
 		ClearGameInfoMap();
 		// Send resubscribe request.
-		boost::shared_ptr<NetPacket> resubscr(new NetPacketResubscribeGameList);
-		GetSender().Send(GetContext().GetSessionData(), resubscr);
+		boost::shared_ptr<NetPacket> packet(new NetPacket(NetPacket::Alloc));
+		packet->GetMsg()->present = PokerTHMessage_PR_subscriptionRequestMessage;
+		SubscriptionRequestMessage_t *netRequest = &packet->GetMsg()->choice.subscriptionRequestMessage;
+		netRequest->subscriptionAction = subscriptionAction_resubscribeGameList;
+		GetSender().Send(GetContext().GetSessionData(), packet);
 		GetContext().SetSubscribeLobbyMsg(true);
 	}
 }
@@ -859,6 +1030,13 @@ ClientThread::SetGameId(unsigned id)
 	m_curGameId = id;
 }
 
+Gsasl *
+ClientThread::GetAuthContext()
+{
+	assert(m_authContext);
+	return m_authContext;
+}
+
 const GameData &
 ClientThread::GetGameData() const
 {
@@ -886,12 +1064,14 @@ ClientThread::SetStartData(const StartData &startData)
 unsigned
 ClientThread::GetGuiPlayerId() const
 {
+	boost::mutex::scoped_lock lock(m_guiPlayerIdMutex);
 	return m_guiPlayerId;
 }
 
 void
 ClientThread::SetGuiPlayerId(unsigned guiPlayerId)
 {
+	boost::mutex::scoped_lock lock(m_guiPlayerIdMutex);
 	m_guiPlayerId = guiPlayerId;
 }
 
@@ -915,9 +1095,9 @@ ClientThread::AddPlayerData(boost::shared_ptr<PlayerData> playerData)
 	{
 		m_playerDataList.push_back(playerData);
 		if (playerData->GetUniqueId() == GetGuiPlayerId())
-			GetCallback().SignalNetClientSelfJoined(playerData->GetUniqueId(), playerData->GetName(), playerData->GetRights());
+			GetCallback().SignalNetClientSelfJoined(playerData->GetUniqueId(), playerData->GetName(), playerData->IsGameAdmin());
 		else
-			GetCallback().SignalNetClientPlayerJoined(playerData->GetUniqueId(), playerData->GetName(), playerData->GetRights());
+			GetCallback().SignalNetClientPlayerJoined(playerData->GetUniqueId(), playerData->GetName(), playerData->IsGameAdmin());
 	}
 }
 
@@ -1106,6 +1286,19 @@ ClientThread::UseServer(unsigned serverId)
 	context.SetAvatarServerAddr(useInfo.avatarServerAddr);
 }
 
+bool
+ClientThread::GetLoginData(LoginData &loginData) const
+{
+	bool retVal = false;
+	boost::mutex::scoped_lock lock(m_loginDataMutex);
+	if (!m_loginData.userName.empty())
+	{
+		loginData = m_loginData;
+		retVal = true;
+	}
+	return retVal;
+}
+
 unsigned
 ClientThread::GetGameIdByName(const std::string &name) const
 {
